
# q-expansion of newform 2800.2.g.c, downloaded from the LMFDB on 15 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2800
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[351, [1, 0]], [2101, [1, 0]], [2577, [-1, 0]], [801, [1, 0]]]
aps_data = [[0, 0], [0, 3], [0, 0], [0, 1], [5, 0], [0, 3], [0, -1], [6, 0], [0, 6], [9, 0], [4, 0], [0, 2], [-4, 0], [0, 10], [0, 1], [0, -4], [-8, 0], [-8, 0], [0, -12], [-8, 0], [0, -2], [13, 0], [0, -4], [-4, 0], [0, -13], [6, 0], [0, 19], [0, 6], [3, 0], [0, -14], [0, 8], [10, 0], [0, -12], [-14, 0], [6, 0], [-5, 0], [0, -2], [0, -10], [0, -3], [0, 1], [-12, 0], [-20, 0], [-3, 0], [0, 4], [0, 8], [8, 0], [11, 0], [0, -5], [0, 1], [-4, 0], [0, -24], [1, 0], [-26, 0], [-30, 0], [0, 14], [0, 2], [-18, 0], [0, 0], [0, 6], [11, 0], [0, -31], [0, 5], [0, 23], [18, 0], [0, 7], [0, -26], [-28, 0], [0, 22], [0, -10], [-10, 0], [0, -3], [-16, 0], [0, 17], [0, 4], [12, 0], [0, 36], [31, 0], [0, 7], [25, 0], [-6, 0], [36, 0], [-19, 0], [9, 0], [0, -2], [-2, 0], [0, -14], [9, 0], [0, 16], [40, 0], [0, -16], [0, -5], [6, 0], [0, -26], [15, 0], [-9, 0], [0, -3], [14, 0], [14, 0], [0, 4], [39, 0], [0, 24], [0, 40], [0, -28], [30, 0], [12, 0], [0, 13], [0, 16], [0, 27], [15, 0], [30, 0], [0, -13], [0, -42], [0, -46], [-10, 0], [-47, 0], [2, 0], [0, 19], [0, 8], [0, -2], [31, 0], [-32, 0], [0, -4], [0, -27], [0, 8], [-28, 0], [-5, 0], [-31, 0], [6, 0], [0, -24], [0, 47], [-5, 0], [0, 36], [19, 0], [0, -32], [-18, 0], [-22, 0], [0, -45], [0, -31], [0, -33], [-9, 0], [-14, 0], [-39, 0], [0, 28], [0, -50], [28, 0], [2, 0], [0, 10], [0, 18], [36, 0], [0, 16], [0, 26], [32, 0], [0, -36], [0, -8], [0, -14], [48, 0], [-31, 0], [-28, 0], [0, -37], [32, 0], [0, -8], [0, -24], [0, 2], [24, 0], [0, -42], [0, -43], [-16, 0], [0, 11], [35, 0], [0, -58], [56, 0], [-46, 0], [-27, 0], [0, -42], [14, 0], [-52, 0], [17, 0], [9, 0], [0, 20], [56, 0], [0, 52], [12, 0], [0, -38], [0, -10], [0, -12], [30, 0], [0, -56], [0, 59], [23, 0], [48, 0], [0, -18], [0, 34], [-25, 0], [-40, 0], [0, 36], [0, 51], [-41, 0], [0, 44], [0, 15], [0, 35], [-46, 0], [58, 0], [0, 18], [6, 0], [-30, 0], [0, -61], [-58, 0], [0, 60], [-54, 0], [64, 0], [0, 36], [42, 0], [0, 4], [0, 1], [-18, 0], [20, 0], [0, 50], [-12, 0], [0, 2], [0, 42], [26, 0], [-58, 0], [27, 0], [0, 6], [0, -71], [-38, 0], [0, 5], [-25, 0], [0, -43], [4, 0], [0, -4], [18, 0], [32, 0], [9, 0], [0, 7], [0, -75], [8, 0], [0, -32], [65, 0], [50, 0], [0, -56], [-56, 0], [0, 15], [-70, 0], [0, 75], [-6, 0], [0, 40], [42, 0], [-19, 0], [0, -22], [0, 10], [18, 0], [0, 36], [-24, 0], [0, -54], [7, 0], [-58, 0], [0, -4], [0, -3], [0, -75], [0, 18], [0, -22], [-4, 0], [0, -65], [0, -78], [14, 0], [39, 0], [2, 0], [0, -68], [0, -42], [-48, 0], [0, -12], [0, -11], [-36, 0], [0, 38], [0, 47], [0, -68], [-50, 0], [33, 0], [-22, 0], [0, -77], [19, 0], [0, 48], [-18, 0], [0, 1], [81, 0], [0, 74], [0, -30], [20, 0], [12, 0], [33, 0], [0, 43], [0, 32], [-68, 0], [0, -66], [72, 0], [2, 0], [0, 83], [-60, 0], [0, -59], [0, -15], [0, -34], [-53, 0]]
